//Setup Debug mode flags
#define DEBUG 1

#if DEBUG == 1
#define debug(x) Serial.print(x)
#define debugln(x) Serial.println(x)
#else
#define debug(x)
#define debugln(x)
#endif

//Setup other defines that will make reusing this code easier later down the road.
#define HOMEWARD FORWARD
#define AWAY BACKWARD
#define STEPTYPE DOUBLE
#define VERSION "1.2"

//Setup Needed Libraries
#include <Wire.h>
#include <Adafruit_MotorShield.h>
#include "LiquidCrystal.h"
Adafruit_MotorShield AFMS = Adafruit_MotorShield(); 
Adafruit_StepperMotor *stepper = AFMS.getStepper(200, 2);
LiquidCrystal lcd(1);

//Setup Constants
const int homeend = A1; //Home-side endstop
const int knob = A3; //Potentiometer
const int button = A2; //GO Button

//Global Variables
int knobval = 0; 
int move_delay = 0;
int stepcount = 0;
int travel = 0;

void setup() {
  Serial.begin(9600);
  //Set Pinmodes
  pinMode(homeend, INPUT_PULLUP);
  pinMode(button, INPUT_PULLUP);

  lcd.begin(16, 2);
  AFMS.begin();
  TWBR = ((F_CPU /400000l) - 16) / 2; // Change the i2c clock to 400KHz
  splash();
  music();
  lcd.clear();
  
  //Home the Machine
  homing();
  lcd.clear();
  lcd.setCursor(0,0);
  ver();
}

void loop() {
  /*
   * This is the main loop of the program, though it's probably the simplest part.
   * It continually rewrites the speed of the slide (expressed as a percentage, though that may change later) on to the screen, waiting until the GO button is pressed.
   */
  debugln("In Void Loop");
  lcd.setCursor(0,1);
  lcd.print("Speed: ");
  pollknob();
  speedprint();
  if (digitalRead(button) == HIGH && digitalRead(homeend) == LOW) {
    debugln("GO"); //If both the GO button and the home endstop are engaged (which it should be) then the it will jump off to the "go" function to begin movement.
    go();
  }
}

void go() {
  /*
   * Okay! Time to move!
   * So, at this point we take the number of steps we counted when moving home from the user's defined endpoint and count up to that from the home position.
   * The delay between steps is continually updated as the move happens depending on the knob position.
   * This has been a sitcking point for me because I don't know if I should do it that way or just set the speed to whatever it was when the go button was pressed.
   * On the one hand, keeping it adjustable during the move allows the user to kind of have a dress rehearsal. To see how the move looks and adjust it in real time 
   * based on what they want.
   * On the other hand, setting it at GO will let me do things like calculate the estimated move time in advance and display it on the screen 
   * and let me simply pass the move speed as a value to the go and gohome functions, maybe even have them as one function that direction and speed could be  
   * passed to.
   * For the moment, I'm going to keep it adjustable.
   */
  lcd.setCursor(0,1);
  lcd.print("Moving...       ");
  travel = 0;
  while (stepcount != travel) {
    pollknob();
    stepper->onestep(AWAY, STEPTYPE);
    delay(move_delay);
    travel++;
  }
  gohome();
  
}
void pollknob() { //Does an analog read and map on the potentiometer knob to set 'knobval' (used to display speed percentage) and 'move_delay' (used to set the millisecond delay between steps).
  knobval = map(analogRead(knob), 1023, 0, 0, 100);
  move_delay = map(analogRead(knob), 1023, 0, 1000, 0);
}

void speedprint(){
  lcd.print(knobval); lcd.print("%    ");
}

void homing() {
/* So, here we have the initial homing of the machine. 
 * Because the slider only has one endstop (on the "home" side) there is no way for the machine to know if it's reached its physical limit.
 * Honestly, with this code, it doesn't have to know. 
 * The first thing it does is move the carraige homeward until it hits the endstop. To do this, it calls the "gohome" function.
 */
  debugln("void homing");
  gohome();
  /* Now we release the stepper motor so that the user can move the carraige to whatever position they want to be the end point. 
   * The machine will wait until the user pushes the button and then it will call "gohome" again.
   */
  stepper->release();
  stepcount = 0;
  debugln("Homing if section");
  lcd.setCursor(0,0);
  lcd.print("Move Carraige to");
  lcd.setCursor(0,1);
  lcd.print("Desired Move End");
  buttonWait();
  gohome();
  return;
}

void gohome() {
  /* As long as the Home Endstop is not engaged, the machined will move the stepper in the direction defined by the "HOMEWARD" define at the top of the code.
   * It will move the carraige one step at a time, keeping track of the steps, with a delay between each as determined by the "pollknob" function. 
   */
  debugln("Void gohome");
  lcd.clear();
  lcd.setCursor(0,1);
  lcd.print("Going Home");
  int stepcountnew = 0;
  while (digitalRead(homeend) == HIGH) { //You can't go home if you're already there!
    pollknob(); //Ping pollknob for most current "move_delay" value
    stepper->onestep(HOMEWARD, STEPTYPE); //Move one step in the direction set by HOMEWARD and in the way set by STEPTYPE
    stepcountnew++; //Increment the steptracker
    delay(move_delay);
  }
  stepcount = stepcountnew; //Save the steps to a globally accessable variable.
  debug(stepcount);
  debugln(" steps");
  return;
}

void ver() {//Prints version information
  lcd.print("Tobogan Mini ");
  lcd.print(VERSION);
  debugln("Tobogan Mini 0.1");
}

void splash() {//Splash Screen
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.setBacklight(HIGH);
  lcd.print("FrostWorks");
  debugln("##########DEBUGMODE###########");
  debugln("FrostWorks");
  lcd.setCursor(0,1);
  ver();
}

void buttonWait(){ //Waits very patiently for the button to be pressed.
  int buttonState = 0;
  debugln("Waiting for Button Press");
  while(1){
    buttonState = digitalRead(button);
    if (buttonState == HIGH) {
      return;
    }
  }
}

void music() { //Play a little tune. Right now it's "In the Hall of the Mountain King"
  play(4,250);
  play(3,250);
  play(2,250);
  play(1,250);
  play(0,250);
  play(2,250);
  play(0,1000);
  delay(2500);
}

void play(int note, int hold){ //Engages and moves the stepper back and forth based on the delay between steps (sets the pitch) and how long to hold the note in milliseconds/
  unsigned long Time = millis();
  while (millis() < Time + hold){
    stepper->onestep(FORWARD, DOUBLE);
    delay(note);
    stepper->onestep(BACKWARD, DOUBLE);
  }
}
